MHiCSClass = MHiCSClass or class()

function MHiCSClass:init()
	self:_setup()
end

function MHiCSClass:_setup()
	if not Global.MHiCS_data then
		Global.MHiCS_data = {
			extra_missions = {{},{},{},{}},
			tmp_extra_missions = {{},{},{},{}}
		}
	end
	self._global = Global.MHiCS_data
	self.crimespree_lobby_everywhere = false
	self._path = ModPath
	self._data_path = SavePath .. "More_Heists_in_Crime_Spree.txt"
	self._instance = ModInstance
	self._network = {
		id = "MHICS",
		sitout = "sitout",
		do_reconn = "do_reconn",
		xtra_begin = "xs",
		xtra_cont = "xc",
		xtra_fin = "xf",
		xtra_split = "*",
		new_obj = "no",
	}
	self.version = self._instance:GetVersion()
	self._strings = {
		dummy = "MHiCS_dummy",
		no_contact = "MHiCS_no_contact",
		dlc = "MHiCS",
	}	
	
	local BLT = rawget(_G, "BLT")
	if (BLT and BLT.Mods) then
		for _, mod in ipairs(BLT.Mods:Mods()) do
			local name = mod:GetId()
			if  name == "UNIfied CrimeNet" then
				self.crimespree_lobby_everywhere = true
			end
		end
	end
	
	self.settings = { news={} }
	self:default_values()
	if not self:Load() then
		self:default_values()
		self:Save()
	end
	self.settings.more = self.settings.more and self.settings.enabled
	local debug_short_add = 5
	local debug_med_add = 7
	local debug_long_add = 10
	-- ghost param: 0 = loud only, 1 = stealth only, 2 = both
	self.heists = {
		-- old stealth heists
		branchbank_cash = {},
		cage = {ghost = 1},
		kosugi = {ghost = 1},
		dark = {ghost = 1},
		firestarter_2 = {},
		hox_3 = {},
		fish = {ghost = 1},
		election_day_2 = {},
		crojob1 = {},
		framing_frame_3 = {},
		arm_for = {},
		friend = {},
		big = {},
		mus = {},
		roberts = {},
		red2 = {},
		pex = {},
		tag = {ghost = 1},
		-- old short heists
		wwh = {},
		rvd_1 = {},
		rvd_2 = {},
		brb = {},
		arm_cro = {},
		help = {},
		arm_und = {},
		arm_hcm = {},
		arm_par = {},
		arm_fac = {},
		chew = {},
		firestarter_1 = {},
		nail = {},
		watchdogs_1_d = {},
		pines = {},
		moon = {},
		spa = {},
		cane = {},
		mia_2 = {},
		bex = {},
		sah = {},
		-- old long heists
		pbr2 = {},
		pal = {},
		flat = {},
		born = {},
		hox_2 = {},
		hox_1 = {},
		welcome_to_the_jungle_2 = {},
		mia_1 = {},
		rat = {},
		pbr = {},
		glace = {},
		run = {},
		man = {},
		dinner = {},
		jolly = {},
		mad = {},
		arena = {},
		-- More heists
		vit = {value = 13},
		family = {value = 6},
		kenaz = {value = 13},
		jewelry_store = {value = 4},
		ukrainian_job = {value = debug_short_add},
		gallery = {value = 4},
		welcome_to_the_jungle_1_d = {value = 4},
		dah = {value = 8},
		nightclub = {value = debug_short_add},
		election_day_1 = {ghost = 1, value = 1},
		--[[
		-- Four stores not added because it can be instantly solved with 1 ecm.
		four_stores = {value = 3}, 
		--]]
		mallcrasher = {value = 4},
		shoutout_raid = {value = 8},
		election_day_3 = {value = 6},
		watchdogs_2_d = {value = 6},
		chill_combat = {value = 12},
		crojob2_d = {value = 14},
		bph = { value = 13},
		nmh = {ghost = 0, value = 13},
		des = {value = 14},
		peta_1 = {value = 14},
		peta_2 = {value = 14},
		mex = {value = 14},
		mex_cooking = {value = debug_med_add},
		fex = {value = debug_long_add},
		chas = {value = debug_short_add},
		sand = {value = 9},
		chca = {value = 8},
		pent = {value = 9},
		ranc = {value = 8},
		trai = {value = 12},
		corp = {value = debug_med_add},
		deep = {value = 12}
	}
	self.heist_names = {}
	self.heist_to_entry = {}
	
	self.missions = {{}, {}, {}, {}}
	-- Keep the following nil to create them from scratch if all preconditions are met
	self.all_levels = nil
	self.level_to_stage = {}

	self.ip_sit_out = {}
	self.me_sit_out = false
	
	self.current_objective = nil
	-- Load Textures
	for _, file in pairs(SystemFS:list(ModPath .. "guis/dlc/MHiCS")) do
		if not DB:has("texture", Idstring("guis/dlc/MHiCS/".. file:gsub(".texture", ""))) then
			DB:create_entry("texture", Idstring("guis/dlc/MHiCS/".. file:gsub(".texture", "")), ModPath .. "guis/dlc/MHiCS/" .. file)
		end
	end
end

function MHiCSClass:Load()
	local file = io.open(self._data_path, "r")
	if file then
		local decoded = json.decode(file:read("*all")) or {}
		if type(decoded) ~= "table" then
			return false
		end
		for k, v in pairs(decoded) do
			self.settings[k] = v
		end
		file:close()
		if type(self.settings.version) == "number" then
			if self.settings.version < 2 then
				self.settings.news.even_more = true
			end
		elseif type(self.settings.version) == "string" then
			local major = tonumber(string.split(self.settings.version, '%.')[1])
			if major and major < 2 then
				self.settings.news.even_more = true
			end
		end
		return self.settings.version == self.version
	end
	return false
end

function MHiCSClass:Save()
	local file = io.open(self._data_path, "w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

function MHiCSClass:reroll_slot(idx)
	local missions = managers.crime_spree:missions()
	local new_mission = self:_reroll_slot(missions, idx)
	if managers.crime_spree:_is_host() then		
		managers.crime_spree:select_mission(false)
		if managers.menu_component:crime_spree_missions_gui() then
			managers.menu_component:crime_spree_missions_gui():randomize_crimespree(idx)
		end
		managers.crime_spree:send_crime_spree_mission_data(idx, new_mission.id, false, true)
	end
end


function MHiCSClass:_reroll_slot(avail_miss, idx)
	local o_miss1 = avail_miss[((idx) % 3) + 1]
	local o_miss2 = avail_miss[((idx + 1) % 3) + 1]
	local new_mission
	local reroll = true
	while reroll do
		new_mission = table.random(tweak_data.crime_spree.missions[idx])
		if self:is_mission_ok(new_mission.level.level_id) then
			reroll = new_mission.id == o_miss1.id or new_mission.id == o_miss2.id
		end
	end
	avail_miss[idx] = new_mission
	return new_mission
end

function MHiCSClass:repair_heists(avail_miss)
	local repaired = {}
	for i=1,3 do
		local old_mission = avail_miss[i] and managers.crime_spree:get_mission(avail_miss[i].id)
		if old_mission and old_mission.id ~= MHiCS._strings.dummy then
			table.insert(repaired, avail_miss[i])
		else
			local new_mission
			local reroll = true
			while reroll do
				new_mission = table.random(tweak_data.crime_spree.missions[i])
				if self:is_mission_ok(new_mission.level.level_id) then
					reroll = false
					for j=1,i-1 do
						if repaired[j].id == new_mission.id then
							reroll = true
						end
					end
				end
			end
			table.insert(repaired, new_mission)
		end
	end
	return repaired
end

function MHiCSClass:check_mission_list(crime_spree_manager)
	crime_spree_manager._global.available_missions = self:repair_heists(crime_spree_manager._global.available_missions)
end

function MHiCSClass:set_correct_missions()
	if self.settings.more and self.settings.enabled then
		tweak_data.crime_spree.missions = self.missions
	else
		tweak_data.crime_spree.missions = self.orig_missions
	end
	self:check_mission_list(managers.crime_spree)
end

function MHiCSClass:default_values()
	if self.settings.enabled == nil then
		self.settings.enabled = true
	end
	if self.settings.only_lobbies == nil then
		self.settings.only_lobbies = false
	end
	if self.settings.more == nil then
		self.settings.more = true
	end
	if self.settings.last_mission_limit == nil then
		self.settings.last_mission_limit = 7
	end
	self.settings.version = self.version
	self.settings.last_missions = {}
	self.settings.extra_heists = self.settings.extra_heists or {}
	self.settings.extra_heists_warning = self.settings.extra_heists_warning or false
end

-- Create a new network key
function MHiCSClass:generate_network_key(originalkey)
	return originalkey .. "_MHiCS_" .. self.settings.version
end

function MHiCSClass:SendMessage(peer_id, msg_id, data)
	local transmit_table = {}
	if type(data) == 'table' then
		transmit_table = table.map_copy(data)
	else
		transmit_table._data = data
	end
	transmit_table._id = msg_id
	LuaNetworking:SendToPeer(peer_id, self._network.id, LuaNetworking:TableToString(transmit_table))
end

function MHiCSClass:send_extra_heists()
	if not LuaNetworking:IsClient() or not managers.crime_spree:is_active() then
		return
	end
	local extra_heists = self:get_extra_missions_params(tweak_data)
	local msg_tbl = {}
	local msg = ""
	for level_id, _ in pairs(extra_heists) do
		local length = string.len(msg)
		-- Assumption: level_id is always smaller than ~70 characters
		if  length == 0 or length > 150 then
			table.insert(msg_tbl, msg)
			msg = level_id
		else
			msg = msg .. self._network.xtra_split .. level_id
		end
	end
	table.insert(msg_tbl, msg)
	self:SendMessage(1, self._network.xtra_begin, msg_tbl[1])
	local idx = 2
	while idx < #msg_tbl do
		self:SendMessage(1, self._network.xtra_cont, msg_tbl[idx])
		idx = idx + 1
	end
	if idx == #msg_tbl then
		self:SendMessage(1, self._network.xtra_fin, msg_tbl[idx])
	else
		-- Table has one entry
		self:SendMessage(1, self._network.xtra_fin, "")
	end	
end

function MHiCSClass:send_current_objective(peer_id)
	if not self.current_objective then
		return
	end

	for ip,_ in pairs(self.ip_sit_out) do
		local peer = managers.network:session():peer_by_ip(ip)
		if peer and (not peer_id or peer:id() == peer_id) then
			self:SendMessage(peer:id(), self._network.new_obj, self.current_objective)
		end
	end
end

function MHiCSClass:update_current_objective( objective )
	if not managers.crime_spree:_is_host() then
		return
	end

	self.current_objective = objective or self.current_objective
	self:send_current_objective()
end

function MHiCSClass:add_peer(peer_id)
	self._global.extra_missions[peer_id] = {}
	self._global.tmp_extra_missions[peer_id] = {}
	if peer_id == 1 then
		self:send_extra_heists()
	end
	self:generate_mission_list(tweak_data)
end

function MHiCSClass:remove_peer(peer_id)
	self._global.extra_missions[peer_id] = {}
	self._global.tmp_extra_missions[peer_id] = {}
	self:generate_mission_list(tweak_data)
end

function MHiCSClass:add_extra_heists(peer_id, levels_str, modifier)
	if modifier == self._network.xtra_begin then
		self._global.tmp_extra_missions[peer_id] = {}
	end
	if levels_str then
		for _, level_id in ipairs(string.split(levels_str, self._network.xtra_split)) do
			self._global.tmp_extra_missions[peer_id][level_id] = true
		end
	end
	if modifier == self._network.xtra_fin then
		self._global.extra_missions[peer_id] = self._global.tmp_extra_missions[peer_id]
		self:generate_mission_list(tweak_data)
	end
end

function MHiCSClass:set_orig_missions(orig_missions)
	self.orig_missions = orig_missions
end

function MHiCSClass:get_orig_missions()
	return self.orig_missions
end

function MHiCSClass:get_missions()
	return self.missions
end
function MHiCSClass:get_mission_entry(level_id)
	return self.heist_to_entry[level_id] and self.heist_to_entry[level_id].mission or nil
end

function MHiCSClass:get_mission_params(level_id)
	return self.heist_to_entry[level_id].params
end

function MHiCSClass:get_level_id(stage_id)
	if not tweak_data.narrative.stages[stage_id] then
		return self._strings.dummy
	end
	return tweak_data.narrative.stages[stage_id].level_id
end

function MHiCSClass:get_mission_entry_by_stage_id(stage_id)
	return self:get_mission_entry(self:get_level_id(stage_id))
end

function MHiCSClass:get_mission_params_by_stage_id(stage_id)
	return self:get_mission_params(self:get_level_id(stage_id))
end


-- Extract only what is necessary from the settings
function MHiCSClass:check_extra_heist_data(tweak_data, level_id)
	local heist_info = {}
	local old_heist_info = self.settings.extra_heists[level_id] or {}
	heist_info.enabled = old_heist_info.enabled or false
	old_heist_info.params = old_heist_info.params or {}
	heist_info.params = {}
	heist_info.params.value = old_heist_info.params.value or 3
	heist_info.params.ghost = old_heist_info.params.ghost
	self:repair_ghost_param(tweak_data, level_id, heist_info.params)
	self.settings.extra_heists[level_id] = heist_info
	return true
end

-- Resolve stages in stacks
-- This is mostly relevant for election day since day 2 can be on different maps
function MHiCSClass:assign_stages_to_jobs_resolve_stacked_stages(job_id, stage, stages_to_jobs)
	if stage.level_id then
		stages_to_jobs[stage] = stages_to_jobs[stage] or {}
		table.insert(stages_to_jobs[stage], job_id)
		return
	end
	-- stacked stage
	for _, wrapped_stage in ipairs(stage) do
		self:assign_stages_to_jobs_resolve_stacked_stages(job_id, wrapped_stage, stages_to_jobs)
	end
end

-- Resolve wrapped jobs
function MHiCSClass:assign_stages_to_job_resolve_wrapper(tweak_data, job_id, wrapped_id, stages_to_jobs, checked_jobs)
	-- already checked this job
	if checked_jobs[wrapped_id] then
		return
	end	
	-- mark this job as checked
	checked_jobs[wrapped_id] = true
	
	local job =  tweak_data.narrative.jobs[wrapped_id]
	if job.job_wrapper then
		for _, wrapped_job in ipairs(job.job_wrapper) do
			self:assign_stages_to_job_resolve_wrapper(tweak_data, job_id, wrapped_job, stages_to_jobs, checked_jobs)
		end
		return
	end
	
	-- Here we should be left with the innermost wrapped job
	if not job.chain then
		return
	end
	for _, stage in ipairs(job.chain) do
		self:assign_stages_to_jobs_resolve_stacked_stages(job_id, stage, stages_to_jobs)
	end
end


-- Resolves the contact for a given job
function MHiCSClass:get_contact(tweak_data, job_id)
	if not job_id then
		return nil
	end
	local contact = tweak_data.narrative.jobs[job_id].contact
	-- Only return the contact if they are actually recognized, e.g. there is a invalid contact named "wip"
	if contact and tweak_data.narrative.contacts[contact] then
		return contact
	end
	return nil
end

-- As the name implies, create matching stage -> job for later lookup of dlc
-- Stage can be matched to more than one job
function MHiCSClass:assign_stages_to_jobs(tweak_data)	
	self.stage_ids_to_job_ids = {}
	local checked_jobs = {}
	local stages_to_jobs = {}
	for job_id, _ in pairs(tweak_data.narrative.jobs) do
		if self:get_contact(tweak_data, job_id) then
			self:assign_stages_to_job_resolve_wrapper(tweak_data, job_id, job_id, stages_to_jobs, checked_jobs)
		end
	end
	
	-- Resolve stage names
	-- This loses some jobs that are deep_clones but honestly, who cares e.g. about the pro jobs.
	for stage_id, stage in pairs(tweak_data.narrative.stages) do
		if stages_to_jobs[stage] then
			self.stage_ids_to_job_ids[stage_id] = stages_to_jobs[stage]
		end
	end
end

-- Levels can be in multiple stages, see branchbank
function MHiCSClass:assign_levels_to_stages(tweak_data)
	self.level_to_stage = {}
	for stage_id, data in pairs(tweak_data.narrative.stages) do
		if data.level_id then
			self.level_to_stage[data.level_id] = self.level_to_stage[data.level_id] or {}
			table.insert(self.level_to_stage[data.level_id], stage_id)
		end
	end
end

function MHiCSClass:repair_ghost_param(tweak_data, level_id, params)
	if params.ghost == nil then
		local level = tweak_data.levels[level_id]
		if level.ghost_required or level.ghost_required_visual then
			params.ghost = 1
		elseif level.ghost_bonus then
			params.ghost = 2
		else
			params.ghost = 0
		end
	end
end

function MHiCSClass:add_mission(tweak_data, mission, params)
	self:repair_ghost_param(tweak_data, mission.level.level_id, params)
	params.assign = {}
	if params.ghost > 0 then
		table.insert(params.assign, 1)
	end
	if params.ghost ~= 1 then
		if mission.add <= 10 then
			table.insert(params.assign, 2)
		end
		if mission.add >= 9 then
			table.insert(params.assign, 3)
		end
	end
	for _, i in pairs(params.assign) do
		table.insert(self.missions[i], mission)
	end
	table.insert(self.heist_names, mission.level.level_id)
end

-- Check the level ids with some generic tests to determine actual playable levels
function MHiCSClass:get_good_level_ids(tweak_data)
	if self.all_levels then
		return self.all_levels
	end
	-- If we cannot do the checks, then forget it
	if not managers or not managers.localization then
		return {}
	end

	self.all_levels = {}
	for _, level_id in ipairs(tweak_data.levels:get_level_index()) do
		if managers.localization:exists(tweak_data.levels[level_id].name_id) and not tweak_data.levels[level_id].is_safehouse and level_id ~= self._strings.dummy then
			table.insert(self.all_levels, level_id)
		end
		-- There could be more checks but I did not find any 100% precise common indicators (especially regarding custom heists)
	end
	return self.all_levels
end

function MHiCSClass:generate_base_missions(tweak_data)
	local all_base_missions = {}
	local inverse_lookup = {}
	-- At least one entry in tweak_data is broken, so we need an inverse lookup
	-- See arm_und has stage_id cage and cage has stage_id cage
	for stage_id, stage in pairs(tweak_data.narrative.stages) do
		inverse_lookup[stage] = stage_id
	end
	for i=1,3 do
		local mission_list = self.orig_missions[i]
		for _, mission in ipairs(mission_list) do
			-- add mission to heists
			local stage_id = inverse_lookup[mission.level]
			if not self.heists[stage_id] then
				-- Should only happen on newly released heists that are instantly included in CS
				self.heists[stage_id] = {}
				if i == 1 then
					self.heists[stage_id].ghost = 1
				end
			end
			-- Failsafe: Mod claims it is stealth only but original list says it is not
			-- just set to loud and stealth possible
			-- This should not occur at the moment
			if i ~=1 and self.heists[stage_id].ghost and self.heists[stage_id].ghost == 1 then
				self.heists[stage_id].ghost = 2
			end
			self.heists[stage_id].no_dlc = true
			all_base_missions[stage_id] = mission
		end
	end
	return all_base_missions
end

-- Create a dummy mission that is displayed when the selected heist is not known
function MHiCSClass:generate_dummy_mission(tweak_data)
	local heist_id = self._strings.dummy
	local mission =  {
		stage_id = heist_id,
		id = heist_id,
		icon = "csm_" .. heist_id,
		add = 0,
		level = {level_id = heist_id}
	}	
	if not tweak_data.levels[heist_id] then
		table.insert(tweak_data.levels._level_index, heist_id)
	end
	tweak_data.levels[heist_id] = {name_id = heist_id}
	return mission
end

-- Creates necessary entries to look up the mission later on
-- Make sure to only call if you are sure this mission can be started
-- A mission cannot be unregisterred
function MHiCSClass:register_mission(mission, params)
	local level_id = mission.level.level_id
	if not self.heist_to_entry[level_id] then
		self.heist_to_entry[level_id] = { mission = mission, params = params or {}}
		table.insert(self.missions[4], mission)
	end
end


-- Add all 100% safe levels to the list
function MHiCSClass:fill_base_heists(tweak_data)
	self:assign_levels_to_stages(tweak_data)
	self:assign_stages_to_jobs(tweak_data)
	local all_base_missions = self:generate_base_missions(tweak_data)
	-- Add the heists to the lists
	for stage_id, params in pairs(self.heists) do
		local mission =  {
			stage_id = stage_id,
			id = stage_id,
			icon = "csm_" .. stage_id,
			add = params.value or 3, -- random value, should never occur anyway
			level = tweak_data.narrative.stages[stage_id]
		}
		local base_mission = all_base_missions[stage_id]
		if base_mission then
			mission.id = base_mission.id
			mission.icon = base_mission.icon
			mission.add = base_mission.add
		end
		self:register_mission(mission, params)
	end
end

-- Add all extra/custom levels to the list
function MHiCSClass:fill_extra_heists(tweak_data)
	-- create mission entry for all other heists
	for _, level_id in ipairs(self:get_good_level_ids(tweak_data)) do
		if not self.heist_to_entry[level_id] then
			if self:check_extra_heist_data(tweak_data, level_id) then
				local stage = nil
				if self.level_to_stage[level_id] then
					stage = tweak_data.narrative.stages[self.level_to_stage[level_id][1]]
				end
				if not stage then
					stage = {
						type = "d",
						type_id = "heist_type_assault",
						level_id = level_id
					}
				end
				local icon = "csm_" .. level_id
				if not tweak_data.hud_icons[icon] then
					icon = "csm_MHiCS_extra_heist"
				end
				local mission = {
					stage_id = level_id,
					id = level_id,
					icon = icon,
					add = self.settings.extra_heists[level_id].params.value,
					level = stage
				}
				self:register_mission(mission, self.settings.extra_heists[level_id].params)
			end
		end
	end
	-- Add dummy heist for unknown custom heists to list
	local mission = self:generate_dummy_mission(tweak_data)
	self:register_mission(mission, {add = 0, ghost = 2})
end

function MHiCSClass:get_extra_heists_list(tweak_data, enabled_only)
	local extra_heist_list = {}
	for _, level_id in ipairs(self:get_good_level_ids(tweak_data)) do
		if self.settings.extra_heists[level_id] and (not enabled_only or self.settings.extra_heists[level_id].enabled) and not self.heists[level_id] then
			extra_heist_list[level_id] = true
		end
	end
	return extra_heist_list
end

function MHiCSClass:get_extra_missions_params(tweak_data, only_me)
	if not managers or not managers.network or not managers.network:session() or managers.network:session():is_client() then
		only_me = true
	end
	local extra_heist_list = self:get_extra_heists_list(tweak_data, true)
	if only_me then
		return extra_heist_list
	end
	for i = 2,4 do
		if managers.network:session():peer(i) then
			for level_id, _ in pairs(extra_heist_list) do
				if not self._global.extra_missions[i][level_id] then
					extra_heist_list[level_id] = nil
				end
			end
		end
	end
	return extra_heist_list
end


function MHiCSClass:generate_mission_list(tweak_data)
	if LuaNetworking:IsClient() or not managers.crime_spree or not managers.crime_spree:is_active() then
		if #self.heist_names > 0 then
			return
		end
	end
	for i = 1,3 do
		self.missions[i] = {}
	end
	self.heist_names = {}
	for stage_id, _ in pairs(self.heists) do
		local level_id = self:get_level_id(stage_id)
		local entry = self.heist_to_entry[level_id]
		self:add_mission(tweak_data, entry.mission, entry.params)
	end
	local extra_missions_param = self:get_extra_missions_params(tweak_data)
	for level_id, _ in pairs(extra_missions_param) do
		local entry = self.heist_to_entry[level_id]
		self:add_mission(tweak_data, entry.mission, entry.params)
	end
end

-- This only works after calling MHiCSClass:fill_base_heists()
-- If the level is in multiple heists, it returns any one job otherwise nil
function MHiCSClass:get_level_job(tweak_data, level_id)
	if not level_id or not self.level_to_stage[level_id] then
		return nil
	end
	local stage_id = self.level_to_stage[level_id][1]
	if not self.stage_ids_to_job_ids[stage_id] then
		return nil
	end
	return self.stage_ids_to_job_ids[stage_id][1]
end

-- This only works after calling MHiCSClass:fill_base_heists()
-- If the level is in multiple heists, it returns any one contact otherwise nil
function MHiCSClass:get_level_contact(tweak_data, level_id)
	local job_id = self:get_level_job(tweak_data, level_id)
	if not job_id then
		return nil
	end
	return self:get_contact(tweak_data, job_id)
end

-- Checks if a single mission can be included in a set of missions
function MHiCSClass:is_mission_ok(level_id)
	if MHiCS:is_dlc_locked(level_id) then
		return false
	end
	for _, miss in ipairs(MHiCS.settings.last_missions) do
		if MHiCS:get_mission_entry(level_id).id == miss then
			return false
		end
	end
	return true
end


-- This only works after calling MHiCSClass:fill_base_heists()
function MHiCSClass:is_dlc_locked(level_id)
	if MHiCS:get_mission_params(level_id).no_dlc then
		return false
	end
	if not managers or not managers.dlc then
		return true
	end
	
	local is_in_one_job = false
	
	local stage_list = self.level_to_stage[level_id] or {}
	for _, stage_id in ipairs(stage_list) do
		local job_list = self.stage_ids_to_job_ids[stage_id] or {}
		for _, job in ipairs(job_list) do
			is_in_one_job = true
			local dlc_id = tweak_data.narrative.jobs[job].dlc
			if not dlc_id or managers.dlc:is_dlc_unlocked(dlc_id) then
				return false
			end
		end
	end
	
	-- We land here if there is no corresponding job or all jobs require dlc not unlocked
	return is_in_one_job
end


function MHiCSClass:modify_history(mission_id)
	local limit = mission_id and self.settings.last_mission_limit - 1 or self.settings.last_mission_limit
	while #self.settings.last_missions > limit do
		table.remove(self.settings.last_missions, 1)
	end
	if mission_id then
		table.insert(self.settings.last_missions, mission_id)
	end
	self:Save()
end


if not _G.MHiCS then
	_G.MHiCS = MHiCSClass:new()
end