-- Failsafe in case your crimespree breaks because of levels
Hooks:PostHook(CrimeSpreeManager, "load", "MHiCS_CrimeSpreeManager_load", function(self, data, version)
	if not MHiCS.settings.enabled then
		local dialog_data = {
			title = managers.localization:text("MHiCS_mod_not_enabled_title"),
			text = managers.localization:text("MHiCS_mod_not_enabled_desc")
		}
		local yes_button = {
			text = managers.localization:text("dialog_yes")
		}
		local no_button = {
			text = managers.localization:text("dialog_no")
		}
		dialog_data.focus_button = 2
		dialog_data.button_list = {
			yes_button,
			no_button
		}
		yes_button.callback_func = MenuCallbackHandler.MHiCS_confirm_reactivate_callback
		managers.system_menu:add_init_show(dialog_data)
	elseif MHiCS.settings.news.even_more then
		local dialog_data = {
			title = managers.localization:text("MHiCS_big_news_title"),
			text = managers.localization:text("MHiCS_even_more_desc")
		}
		local show_button = {
			text = managers.localization:text("MHiCS_even_more_show_me")
		}
		local dont_show_button = {
			text = managers.localization:text("MHiCS_even_more_dont_show_me")
		}
		dialog_data.focus_button = 1
		dialog_data.button_list = {
			show_button,
			dont_show_button
		}
		show_button.callback_func = MenuCallbackHandler.MHiCS_show_even_more
		dont_show_button.callback_func = MenuCallbackHandler.MHiCS_dont_show_even_more
		managers.system_menu:add_init_show(dialog_data)
	end
	if not self:in_progress() then
		return
	end
	MHiCS:check_mission_list(self)
end)

Hooks:PreHook(CrimeSpreeManager, "apply_matchmake_attributes", "MHiCS_CrimeSpreeManager_apply_matchmake_attributes", function(self, lobby_attributes)
	if self:in_progress() and self:is_active() and MHiCS.settings.enabled and MHiCS.settings.more then
		lobby_attributes.skirmish_wave = 99
	end
end)



Hooks:PreHook(CrimeSpreeManager, "missions", MHiCS_CrimeSpreeManager_missions, function(self)
	MHiCS:check_mission_list(self)
end)

local function MHiCS_find_fit(random_set, to_fit, idx)
	if idx > 3 then
		return MHiCS:get_mission_entry(to_fit[2]).add < MHiCS:get_mission_entry(to_fit[3]).add
	end
	for _, i in pairs(MHiCS:get_mission_params(random_set[idx]).assign) do
		if to_fit[i] == MHiCS._strings.dummy then
			to_fit[i] = random_set[idx]
			if MHiCS_find_fit(random_set, to_fit, idx + 1) then
				return true
			end
			to_fit[i] = MHiCS._strings.dummy
		end
	end			
	return false
end

Hooks:PreHook(CrimeSpreeManager, "on_entered_lobby", "MHiCS_CrimeSpreeManager_on_entered_lobby", function(self)
	if self:_is_host() then
		MHiCS:generate_mission_list(tweak_data)
		MHiCS:set_correct_missions()
	else
		-- Crash protection if you do not know the heist and join lobby instead of game
		if not self._global.server_missions or #self._global.server_missions < 3 then
			self._global.server_missions = self:get_random_missions({})
		end	
		MHiCS:send_extra_heists()
	end
end)

local MHiCS_old_get_mission = CrimeSpreeManager.get_mission
function CrimeSpreeManager:get_mission(mission_id)
	local res = MHiCS_old_get_mission(self, mission_id)
	if not res and MHiCS.settings.enabled then
		res = MHiCS:get_mission_entry(MHiCS._strings.dummy)
	end
	return res
end


local MHiCS_old_get_random_missions = CrimeSpreeManager.get_random_missions
function CrimeSpreeManager:get_random_missions(prev_missions)
	if not MHiCS.settings.enabled or (not MHiCS.settings.more and not Global.game_settings.single_player) then
		return MHiCS_old_get_random_missions(self, prev_missions)
	end
	local is_ok = false
	local result_set = {MHiCS._strings.dummy, MHiCS._strings.dummy, MHiCS._strings.dummy}
	while not is_ok do
		local random_set = {
			table.random(MHiCS.heist_names),
			table.random(MHiCS.heist_names),
			table.random(MHiCS.heist_names)
		}
		is_ok = true
		-- check if missions on their own are ok
		for i=1,3 do
			if not MHiCS:is_mission_ok(random_set[i]) then
				is_ok = false
			end
		end
		-- check for same missions
		if is_ok then
			is_ok = false
			if random_set[1] ~= random_set[2] and random_set[2] ~= random_set[3] and random_set[3] ~= random_set[1] then
				-- check if missions can be fit
				is_ok = MHiCS_find_fit(random_set, result_set, 1)
			end
		end			
	end
	return {
		MHiCS:get_mission_entry(result_set[1]),
		MHiCS:get_mission_entry(result_set[2]),
		MHiCS:get_mission_entry(result_set[3]),
	}
end

Hooks:PreHook(CrimeSpreeManager, "on_mission_completed", "MHiCS_CrimeSpreeManager_on_mission_completed", function(self, mission_id)
	if MHiCS.settings.enabled and self:is_active() and (not self:has_failed()) and self:_is_host() then
		MHiCS:modify_history(mission_id)
	end
end)