---------------------------------------------- Networking ------------------------------------------------
Hooks:Add("NetworkReceivedData", "MHiCS_NetworkReceivedData", function(sender, id, data)
	if id ~= MHiCS._network.id then
		return
	end
	local mod_name = "[" .. managers.localization:text("MHiCS_main_menu_title") .. "]"
	local transmit_table = LuaNetworking:StringToTable(data)
	
	local function sitout()
		local peer = managers.network:session():peer(sender)

		if managers.crime_spree then
			for idx, mission_data in ipairs(managers.crime_spree:missions()) do
				managers.crime_spree:send_crime_spree_mission_data(idx, mission_data.id, mission_data.id == managers.crime_spree._global.current_mission, false)
			end
			for _, modifier_data in ipairs(managers.crime_spree:active_modifiers()) do
				managers.crime_spree:send_crime_spree_modifier(peer, modifier_data, false)
			end
		end
		
		local notify_func = function()
			if managers and managers.chat then
				managers.chat:_receive_message(1, mod_name, managers.localization:text("MHiCS_cannot_join_custom", {name = peer:name()}) , Color.yellow)
			else
				DelayedCalls:Add( "MHiCS_notify_cannot_join" .. peer:name(), 1, notify_func )
			end
		end
		if not MHiCS.ip_sit_out[peer:ip()] then
			DelayedCalls:Add( "MHiCS_notify_cannot_join" .. peer:name(), 1, notify_func )
		end
		MHiCS.ip_sit_out[peer:ip()] = true
		MHiCS:send_current_objective(sender)
	end
	
	local function perform_reconnect()
		if sender ~= 1 then
			-- Someone other than host asked us to perform a reconnect
			return
		end
		
		if not MHiCS.me_sit_out then
			-- We are not sitting this one out
			return
		end
		
		local room_id = managers.network.matchmake.lobby_handler and managers.network.matchmake.lobby_handler:id()
		if not room_id then
			return
		end
		MenuCallbackHandler:_dialog_leave_lobby_yes()
		local dialog_data = {
			title = managers.localization:text("MHiCS_perform_reconnect"),
			text = managers.localization:text("dialog_wait"),
			id = "MHiCS_perform_reconnect_diag",
			no_buttons = true,
			indicator = true
		}

		managers.system_menu:show(dialog_data)
		DelayedCalls:Add( "MHiCS_perform_reconnect", 5, function()
			managers.system_menu:close("MHiCS_perform_reconnect_diag")
			managers.network.matchmake:join_server(room_id, true)
		end)
	end
	
	local function get_custom_heist_objective()
		if not MHiCS.me_sit_out then
			-- We are not sitting this one out, no need to display
			return
		end
		managers.chat:_receive_message(1, mod_name, managers.localization:text("MHiCS_new_objective", {objective = transmit_table._data}) , Color.yellow)
	end
	
	local function xtra_heist_modify()
		MHiCS:add_extra_heists(sender, transmit_table._data, transmit_table._id)
	end
	
	local func_table = {}
	func_table[MHiCS._network.sitout] = sitout
	func_table[MHiCS._network.do_reconn] = perform_reconnect
	func_table[MHiCS._network.xtra_begin] = xtra_heist_modify
	func_table[MHiCS._network.xtra_cont] = xtra_heist_modify
	func_table[MHiCS._network.xtra_fin] = xtra_heist_modify
	func_table[MHiCS._network.new_obj] = get_custom_heist_objective

	local to_call = func_table[transmit_table._id]
	if to_call then
		to_call()
	else
		log("Unknown id " .. transmit_table._id)
	end
end)


---------------------------------------------- Localization ------------------------------------------------
Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_MHiCS", function(loc)
	local loc_lang = BLT.Localization._current
	if loc_lang == 'cht' or BLT.Localization._current == 'zh-cn' then
		loc_lang = "ch"
	end
	for _, filename in pairs(file.GetFiles(MHiCS._path .. "loc/")) do
		local str = filename:match('^(.*).txt$')
		if str and loc_lang == str then
			loc:load_localization_file(MHiCS._path .. "loc/" .. filename)
			break
		end
	end

	loc:load_localization_file(MHiCS._path .. "loc/en.txt", false)
end)

----------------------- Callbacks ----------------------- 
local function MHiCS_impossible_selection()
	local dialog_data = {
		title = managers.localization:text("dialog_error_title"),
		text = managers.localization:text("MHiCS_about_to_imposs_desc")
	}
	local ok_button = {
		text = managers.localization:text("dialog_ok")
	}
	dialog_data.button_list = {
		ok_button
	}
	managers.system_menu:show(dialog_data)
end

Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_MHiCS", function(menu_manager)
	-- Dirty = need to generate the list again
	-- Resend = list of heists I want to participate changed; automatically applies dirty
	local dirty = false
	local resend = false
	MenuCallbackHandler.MHiCS_mod_back = function(this, item)
		dirty = dirty or resend
		if dirty then
			MHiCS:generate_mission_list(tweak_data)
			-- update potential buttons
			if managers.menu_component:crime_spree_missions_gui() then
				managers.menu_component:crime_spree_missions_gui():update_mission()
			end
			dirty = false
		end
		if resend then
			MHiCS.settings.extra_heists_warning = true
			MHiCS:send_extra_heists()
			resend = false
		end
		MHiCS:Save()
	end
	
	MenuCallbackHandler.MHiCS_mod_extra_toggle_enable = function(this, item)
		local level_id = item._parameters["name"]:gsub("MHiCS_toggle_extra_","")
		local value = item:value() == "on" and true or false
		MHiCS.settings.extra_heists[level_id].enabled = value
		resend = true
	end
	
	MenuCallbackHandler.MHiCS_mod_extra_multi_points = function(this, item)
		local level_id = item._parameters["name"]:gsub("MHiCS_multi_extra_points_","")
		MHiCS.settings.extra_heists[level_id].params.value = item:value()
		MHiCS:get_mission_entry(level_id).add = item:value()
		dirty = true
	end
	
	MenuCallbackHandler.MHiCS_mod_extra_multi_approach = function(this, item)
		local level_id = item._parameters["name"]:gsub("MHiCS_mutli_extra_","")
		MHiCS.settings.extra_heists[level_id].params.ghost = item:value()
		dirty = true
	end
	
	MenuCallbackHandler.MHiCS_first_time_extra = function(this,item)
		if MHiCS.settings.extra_heists_warning then
			return
		end
		local dialog_data = {
			title = managers.localization:text("dialog_warning_title"),
			text = managers.localization:text("MHiCS_first_time_extra_desc")
		}
		local ok_button = {
			text = managers.localization:text("dialog_ok")
		}
		dialog_data.button_list = {
			ok_button
		}
		managers.system_menu:show(dialog_data)
	end
	
	MenuCallbackHandler.MHiCS_settings_changed_callback = function(this,item)
		local name = tostring(item._parameters["name"]):gsub("MHiCS_","")
		MHiCS.settings[name] = Utils:ToggleItemToBoolean(item)
		MHiCS:Save()
	end

	MenuCallbackHandler.MHiCS_deactivate_callback = function(this,item)
		if managers.crime_spree.is_active() then
			MHiCS_impossible_selection()
		else
			local dialog_data = {
				title = managers.localization:text("dialog_warning_title"),
				text = managers.localization:text("MHiCS_about_to_deact_desc")
			}
			local yes_button = {
				text = managers.localization:text("dialog_yes")
			}
			local no_button = {
				text = managers.localization:text("dialog_no")
			}
			dialog_data.focus_button = 2
			dialog_data.button_list = {
				yes_button,
				no_button
			}
			yes_button.callback_func = MenuCallbackHandler.MHiCS_confirm_deactivate_callback
			managers.system_menu:show(dialog_data)
		end
	end

	MenuCallbackHandler.MHiCS_confirm_deactivate_callback = function(this,item)
		local menu = MenuHelper:GetMenu("MHiCS_main_menu")
		for _, item in pairs (menu._items) do
			if item._parameters.name == "MHiCS_deactivate" then
				item:set_enabled(false)
			end
			if item._parameters.name == "MHiCS_reactivate" then
				item:set_enabled(true)
			end
		end
		MHiCS.settings.enabled = false
		MHiCS.settings.more = false
		MHiCS:set_correct_missions()
		MenuCallbackHandler:save_progress()
		MHiCS:Save()
	end

	MenuCallbackHandler.MHiCS_reactivate_callback = function(this,item)
		if managers.crime_spree.is_active() then
			MHiCS_impossible_selection()
		else
			local dialog_data = {
				title = managers.localization:text("dialog_warning_title"),
				text = managers.localization:text("MHiCS_about_to_react_desc")
			}
			local yes_button = {
				text = managers.localization:text("dialog_yes")
			}
			local no_button = {
				text = managers.localization:text("dialog_no")
			}
			dialog_data.focus_button = 2
			dialog_data.button_list = {
				yes_button,
				no_button
			}
			yes_button.callback_func = MenuCallbackHandler.MHiCS_confirm_reactivate_callback
			managers.system_menu:show(dialog_data)
		end
	end

	MenuCallbackHandler.MHiCS_confirm_reactivate_callback = function(this,item)
		local menu = MenuHelper:GetMenu("MHiCS_main_menu")
		for _, item in pairs (menu._items) do
			if item._parameters.name == "MHiCS_deactivate" then
				item:set_enabled(true)
			end
			if item._parameters.name == "MHiCS_reactivate" then
				item:set_enabled(false)
			end
		end
		MHiCS.settings.enabled = true
		MHiCS.settings.more = true
		MHiCS:set_correct_missions()
		MHiCS:Save()
	end
	
	MenuCallbackHandler.MHiCS_show_even_more = function(this,item)
		if Steam then
			Steam:overlay_activate("url", "https://modworkshop.net/mod/24452/")
		end	
		MenuCallbackHandler.MHiCS_dont_show_even_more(this, item)
	end
	
	MenuCallbackHandler.MHiCS_dont_show_even_more = function(this, item)
		MHiCS.settings.news.even_more = false
		MHiCS:Save()
	end
	
	MenuCallbackHandler.MHiCS_reroll_mission_slot = function(this, item)
		MHiCS.settings.news.even_more = false
		MHiCS:Save()
	end
	
	MenuCallbackHandler.MHiCS_set_last_mission_limit = function(this, item)
		MHiCS.settings.last_mission_limit = item:value()
		MHiCS:modify_history()
	end
end)

----------------------- Menu Creation ----------------------- 

local extra_contact_names = {}

Hooks:Add("MenuManagerSetupCustomMenus", "Base_SetupCustomMenus_Json_MHiCS_menu", function( menu_manager, nodes )
	MenuHelper:NewMenu( "MHiCS_main_menu" )
	MenuHelper:NewMenu( "MHiCS_extra_heists" )
	MHiCS:fill_base_heists(tweak_data)
	MHiCS:fill_extra_heists(tweak_data)
	MHiCS:generate_mission_list(tweak_data)
	extra_contact_names = {}
	for level_id, _ in pairs(MHiCS:get_extra_heists_list(tweak_data)) do
		local contact = MHiCS:get_level_contact(tweak_data, level_id) or MHiCS._strings.no_contact
		extra_contact_names[contact] = extra_contact_names[contact] or {}
		table.insert(extra_contact_names[contact], level_id)
	end
	for contact, _ in pairs(extra_contact_names) do
		MenuHelper:NewMenu( "MHiCS_contact_menu_" .. contact )
	end
end)

Hooks:Add("MenuManagerBuildCustomMenus", "Base_BuildCustomMenus_Json_MHiCS_menu", function( menu_manager, nodes )
	local parent_menu = "blt_options"
	local menu_id = "MHiCS_main_menu"
	local menu_name = "MHiCS_main_menu_title"
	local menu_desc = "MHiCS_main_menu_desc"

	nodes[menu_id] = MenuHelper:BuildMenu( menu_id, {} )

	MenuHelper:AddMenuItem( nodes[parent_menu], menu_id, menu_name, menu_desc, nil )
	
	parent_menu = menu_id
	local menu_id = "MHiCS_extra_heists"
	nodes[menu_id] = MenuHelper:BuildMenu( menu_id, {back_callback="MHiCS_mod_back"} )
	parent_menu = menu_id
	
	for contact, _ in pairs(extra_contact_names) do
		menu_id = "MHiCS_contact_menu_" .. contact
		nodes[menu_id] = MenuHelper:BuildMenu( menu_id, {} )
	end
end)

Hooks:Add("MenuManagerPopulateCustomMenus", "Base_PopulateCustomMenus_Json_MHiCS_menu", function( menu_manager, nodes )
	local menu_id = "MHiCS_main_menu"
	MenuHelper:AddButton({
		id = "MHiCS_deactivate",
		title = "MHiCS_deactivate_button_title",
		desc = "MHiCS_deactivate_button_desc",
		callback = "MHiCS_deactivate_callback",
		menu_id = menu_id,
		priority = 5,
		localized = true,
		disabled = not MHiCS.settings.enabled
	})
	MenuHelper:AddButton({
		id = "MHiCS_reactivate",
		title = "MHiCS_reactivate_button_title",
		desc = "MHiCS_reactivate_button_desc",
		callback = "MHiCS_reactivate_callback",
		menu_id = menu_id,
		priority = 4,
		localized = true,
		disabled = MHiCS.settings.enabled
	})
	
	MenuHelper:AddDivider({
		id = "MHiCS_main_menu_div1",
		menu_id = menu_id,
		priority = 3,
	})
	
	local last_mission_options = {}
	for i=1,25 do
		table.insert(last_mission_options, tostring(i))
	end
	
	MenuHelper:AddMultipleChoice({
		id = "MHiCS_main_menu_mission_limit_choice",
		title =  managers.localization:text("MHiCS_last_mission_limit"),
		desc = managers.localization:text("MHiCS_last_mission_limit_desc"),
		items = last_mission_options,
		value = MHiCS.settings.last_mission_limit,
		callback = "MHiCS_set_last_mission_limit",
		menu_id = menu_id,
		localized = false,
		localized_items = false,
		priority = 2,
	})
	
	MenuHelper:AddDivider({
		id = "MHiCS_main_menu_div2",
		menu_id = menu_id,
		priority = 1,
	})

	MenuHelper:AddButton({
		id = "MHiCS_extra_heists_button",
		title = "MHiCS_extra_heists_title",
		desc = "MHiCS_extra_heists_desc",
		callback = "MHiCS_first_time_extra",
		next_node = "MHiCS_extra_heists",
		menu_id = menu_id,
		priority = 0,
		localized = true,
	})
	
	-- Sort contacts alphabetically by name
	local function contact_sort(c_a, c_b)
		local name_a = tweak_data.narrative.contacts[c_a] and tweak_data.narrative.contacts[c_a].name_id
		local name_b = tweak_data.narrative.contacts[c_b] and tweak_data.narrative.contacts[c_b].name_id
		-- handle nil
		if not name_a then
			return name_b ~= nil
		end
		if not name_b then
			return false
		end
		return managers.localization:text( name_a ) < managers.localization:text( name_b )
	end
	
	-- Sort levels alphabetically by name
	local function level_sort(l_a, l_b)
		local name_a = tweak_data.levels[l_a] and tweak_data.levels[l_a].name_id
		local name_b = tweak_data.levels[l_b] and tweak_data.levels[l_b].name_id
		-- handle nil
		if not name_a then
			return name_b ~= nil
		end
		if not name_b then
			return false
		end
		return managers.localization:text( name_a ) < managers.localization:text( name_b )
	end
		
	
	local parent_menu = "MHiCS_extra_heists"
	for contact, heist_list in table.sorted_map_iterator(extra_contact_names, contact_sort) do
		menu_id = "MHiCS_contact_menu_" .. contact
		local contact_name = managers.localization:text("MHiCS_contact_no_contact")
		if tweak_data.narrative.contacts[contact] then
			contact_name = managers.localization:text(tweak_data.narrative.contacts[contact].name_id)
		end
		MenuHelper:AddButton({
			id = "MHiCS_extra_heists_contact_" .. contact .. "_button",
			title = contact_name,
			desc = managers.localization:text("MHiCS_contact_descr", {contact = contact_name}),
			next_node = menu_id,
			menu_id = parent_menu,
			localized = false,
		})
		
		local count = 4 * #heist_list
		local points_multiple_choice = {}
		for i=1,15 do
			table.insert(points_multiple_choice, tostring(i))
		end
		
		table.sort(heist_list, level_sort)
		for _, level_id in ipairs(heist_list) do
			local name = managers.localization:text(tweak_data.levels[level_id].name_id)
			local title = name
			local job_id = MHiCS:get_level_job(tweak_data, level_id)
			if job_id then
				local job_name = managers.localization:text(tweak_data.narrative.jobs[job_id].name_id)
				if job_name ~= title then
					title = title .. " (" .. job_name .. ")"
				end
			end
			MenuHelper:AddToggle({
				id = "MHiCS_toggle_extra_" .. level_id,
				title = title,
				desc = managers.localization:text("MHiCS_extra_heist_enable", {heist=name}),
				value = MHiCS.settings.extra_heists[level_id].enabled,
				callback = "MHiCS_mod_extra_toggle_enable",
				menu_id = menu_id,
				localized = false,
				priority = count + 3,
			})
			MenuHelper:AddMultipleChoice({
				id = "MHiCS_multi_extra_points_" .. level_id,
				title =  managers.localization:text("MHiCS_extra_heists_points"),
				desc = managers.localization:text("MHiCS_extra_heist_points_desc", {heist=name}),
				items = points_multiple_choice,
				value = MHiCS.settings.extra_heists[level_id].params.value,
				callback = "MHiCS_mod_extra_multi_points",
				menu_id = menu_id,
				localized = false,
				localized_items = false,
				priority = count + 2,
			})
			MenuHelper:AddMultipleChoice({
				id = "MHiCS_mutli_extra_" .. level_id,
				title =  managers.localization:text("MHiCS_extra_heist_approach"),
				desc = managers.localization:text("MHiCS_extra_heist_approach_desc", {heist=name}),
				items = {"MHiCS_extra_heist_approach_opt_stealth", "MHiCS_extra_heist_approach_opt_both", "MHiCS_extra_heist_approach_opt_loud"},
				item_values = {1,2,0},
				show_value = true,
				value = MHiCS.settings.extra_heists[level_id].params.ghost,
				callback = "MHiCS_mod_extra_multi_approach",
				menu_id = menu_id,
				localized = false,
				localized_items = true,
				priority = count + 1,
			})
			MenuHelper:AddDivider({
				id = "MHiCS_main_menu_div".. tostring(i),
				menu_id = menu_id,
				size = 24,
				priority = count,
			})
			
			count = count - 4
		end
	end
end)

local function MHiCS_remove_only_MHiCS(node)
	for i=1,#node._items do
		if node._items[i]._parameters.name == "MHiCS_only_lobbies" then
			table.remove(node._items, i)
			return
		end
	end
end

local function MHiCS_insert_only_MHiCS(node)
	for _, it in pairs(node._items) do
		if it._parameters.name == "MHiCS_only_lobbies" then
			return
		end
	end
	local data = {
		type = "CoreMenuItemToggle.ItemToggle",
		{
			_meta = "option",
			icon = "guis/textures/menu_tickbox",
			value = "on",
			x = 24,
			y = 0,
			w = 24,
			h = 24,
			s_icon = "guis/textures/menu_tickbox",
			s_x = 24,
			s_y = 24,
			s_w = 24,
			s_h = 24
		},
		{
			_meta = "option",
			icon = "guis/textures/menu_tickbox",
			value = "off",
			x = 0,
			y = 0,
			w = 24,
			h = 24,
			s_icon = "guis/textures/menu_tickbox",
			s_x = 0,
			s_y = 24,
			s_w = 24,
			s_h = 24
		}
	}
	local params = {
		name = "MHiCS_only_lobbies",
		text_id = "MHiCS_toggle_only_title",
		help_id = "MHiCS_toggle_only_desc",
		callback = "MHiCS_settings_changed_callback",
		localize = true
	}

	local item = node:create_item(data, params)
	item:set_value(MHiCS.settings.only_lobbies and "on" or "off")
	item:set_callback_handler(node.callback_handler)
	for i=1,#node._items do
		if node._items[i]._parameters.name == "toggle_friends_only" then
			table.insert(node._items, i + 1, item)
			return
		end
	end
end

local MHiCS_old_crimenetfilters_modify_node = MenuCrimeNetFiltersInitiator.modify_node

function MenuCrimeNetFiltersInitiator:modify_node(original_node, data)
	local node = MHiCS_old_crimenetfilters_modify_node(self, original_node, data)
	if (Global.game_settings.gamemode_filter == GamemodeCrimeSpree.id or MHiCS.crimespree_lobby_everywhere) and MHiCS.settings.enabled then
		MHiCS_insert_only_MHiCS(node)
	else
		MHiCS_remove_only_MHiCS(node)
	end		
	return node
end